/*
	main.c
 
    This file is part of pdd (Palm dd).
 
    A memory imaging tool for forensic analysis of Palm OS platform devices

    Copyright (c) 2002 Joe Grand
*/

#include "pdd.h"

char copyright[] = "Copyright (c) 2002 Joe Grand [jgrand@mindspring.com]";

int main (int argc, char *argv[])
{
	int c;
    char *param, *delim;
	
    unsigned short  card;
    MemoryType      type;
	unsigned char   port[FILENAME_MAX];
    unsigned char   of[FILENAME_MAX];

	/* set defaults */
	strcpy (port, "COM1");
    strcpy (of, "");
    card = 0;
	type = RAM;

	if (argc > 1)
	{
		/* if first option is invalid */
		if (strstr (argv[1], "if=") == NULL &&
            strstr (argv[1], "of=") == NULL && 
            strstr (argv[1], "card=") == NULL && 
            strstr (argv[1], "type=") == NULL &&
			strstr (argv[1], "-h") == NULL && 
			strstr (argv[1], "-?") == NULL)
 			    usage(argv[0]);

		/* check options */
		for (c = 1; c < argc; c++)
		{
            if ((delim = strstr(argv[c], "if=")) != NULL) /* port */
            {
                param = delim + 3;
                if (*param) strncpy (port, param, FILENAME_MAX-1);
                else
                    usage(argv[0]);
            }
            else if ((delim = strstr(argv[c], "of=")) != NULL) /* output file */
            {
                param = delim + 3;
                if (*param) strncpy (of, param, FILENAME_MAX-1);
                else
                    usage(argv[0]);
            }
			else if ((delim = strstr(argv[c], "card=")) != NULL) /* card number */
			{
                param = delim + 5;
                if (*param) card = atoi(param); // returns 0 if invalid
			    if ((card == 0 && *param != '0'))
				    usage(argv[0]);
            }
            else if ((delim = strstr(argv[c], "type=")) != NULL) /* memory type */
			{
                param = delim + 5;
                if (_stricmp (param, "RAM") == 0)
				    type = RAM;
			    else if (_stricmp (param, "ROM") == 0)
    			    type = ROM;
			    else
	    		    usage(argv[0]);
            }
		    else if (strstr (argv[c], "-h") != NULL || 
                     strstr (argv[c], "-?") != NULL) /* help */
				help(); 
		}
	}

    signal(SIGINT, (fnSignal) wrapup); // set Ctrl-C signal handler
	pdd (port, of, card, type); /* finally invoke the real routine */

	return 0;
}


void usage (char *argv)
{
	fprintf(stderr, "\nUsage: %s [ operand=value ... ] (-? for help)\n", argv);
	exit(0);
}


void help (void)
{
	fprintf(stderr, "\nJoe Grand [jgrand@mindspring.com], June 2002 v1.11");

	fprintf(stderr, "\n\n\nNAME");
	fprintf(stderr, "\n\tpdd - Memory imaging tool for forensic analysis of Palm OS");
	fprintf(stderr, "\n\tplatform devices");
	
	fprintf(stderr, "\n\nUSAGE");
	fprintf(stderr, "\n\tpdd  [ operand=value ... ]");

    fprintf(stderr, "\n\n\tPlace the Palm device in the cradle or attach to a HotSync");
    fprintf(stderr, "\n\tcable connected to a serial port on the desktop PC. The Palm");
    fprintf(stderr, "\n\tOS Console Mode must be enabled.");

	fprintf(stderr, "\n\nOPERANDS");
    fprintf(stderr, "\n\tif=value     : name of serial communications port [default = COM1]");
	fprintf(stderr, "\n\tof=value     : name of output file [default = standard output]");
    fprintf(stderr, "\n\tcard=n       : Palm card number [default = 0 for built-in card]");
	fprintf(stderr, "\n\ttype=value   : Palm card memory type; RAM or ROM [default = RAM]");
    fprintf(stderr, "\n\t-? or -h     : help [you're reading it]");

    fprintf(stderr, "\n\n");
	exit(0);
}
