/*
	pdd.h
 
    This file is part of pdd (Palm dd).
 
    A memory imaging tool for forensic analysis of Palm OS platform devices

    Copyright (c) 2002 Joe Grand

    Portions of this file are Copyright (c) 1995-1999 Palm Computing, Inc. 
    or its subsidiaries.
*/

#ifndef __INC_PDD_H__
#define __INC_PDD_H__

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <windows.h>
#include <time.h>
#include <fcntl.h>
#include <io.h>
#include <signal.h>

#pragma pack(push,1) // align to 1-byte boundaries

/* GENERAL DEFINES */

// byteswap (motorola -> intel)
#define bswap16(x)		((unsigned short)x = ((unsigned short)_lrotl (x,8) & 0xff00) | ((unsigned short)_lrotr (x,8) & 0x00ff))
#define bswap32(x)		((unsigned int)x = ((unsigned int)_lrotl (x,8) & 0x00ff00ff) | ((unsigned int)_lrotr (x,8) & 0xff00ff00))

#define MAX_BUF_SIZE	(sysPktMaxBodySize + 12)	/* maximum length of receive buffer */
#define DEF_TIMEOUT		500	 						/* default serial timeout, in ms */
#define FAST_TIMEOUT	70	 						/* fast serial timeout, in ms */

typedef void (*fnSignal)(int);

typedef enum
{
	RAM = 0,
	ROM
} MemoryType;

/* "Rom Transfer.c" has information on determining base addresses by reading 
 * DragonBall chip select registers. Ours are hard-coded based on defined standards.
 */
#define	hwrCardBase		    0x10000000L	// DragonBall 68328
#define	hwrCardBaseEZ		0x00000000L	// DragonBall 68EZ328
#define	hwrCardBaseVZ		0x00000000L	// DragonBall 68VZ328

#define	hwrCardSize			0x10000000L	// Size of each card slot
#define	hwrCardSizeEZ		0x20000000L
#define	hwrCardSizeVZ		0x20000000L

#define	hwrROMBase		    0x10C00000L	// Base of ROM
#define	hwrROMBaseEZ		0x10C00000L
#define	hwrROMBaseVZ		0x10000000L

#define hwrcsASelect0               0xFFFFF110L // Base of Chip-Select Register
#define hwrChipSelectSize64k        0x00
#define hwrChipSelectSize128k       0x01
#define hwrChipSelectSize256k       0x03
#define hwrChipSelectSize512k       0x07
#define hwrChipSelectSize1M         0x0F
#define hwrChipSelectSize2M         0x1F
#define hwrChipSelectSize4M         0x3F
#define hwrChipSelectSize8M         0x7F
#define hwrChipSelectSize16M        0xFF

#define hwrEZcsASelect              0xFFFFF110L
#define	hwrEZChipSelectSizeMask     0x0E
#define hwrEZChipSelectSize128k     0x0
#define hwrEZChipSelectSize256k     0x2
#define hwrEZChipSelectSize512k     0x4
#define hwrEZChipSelectSize1M       0x6
#define hwrEZChipSelectSize2M       0x8
#define hwrEZChipSelectSize4M       0xA
#define hwrEZChipSelectSize8M       0xC
#define hwrEZChipSelectSize16M      0xE

#define hwrVZcsASelect              0xFFFFF110L
#define	hwrVZChipSelectSizeMask     0x0E
#define hwrVZChipSelectSize128k     0x0
#define hwrVZChipSelectSize256k     0x2
#define hwrVZChipSelectSize512k     0x4
#define hwrVZChipSelectSize1M       0x6
#define hwrVZChipSelectSize2M       0x8
#define hwrVZChipSelectSize4M       0xA
#define hwrVZChipSelectSize8M       0xC
#define hwrVZChipSelectSize16M      0xE


/* SystemPkt.h */
#define	sysPktMaxMemChunk	256 // Max # of bytes we can read/write at a time with the ReadMem and WriteMem commands
#define	sysPktMaxBodySize	(sysPktMaxMemChunk+16) // Maximum body size for System Packets

/* SystemMgr.h */
#define	sysROMTokenSnum			'snum'	// Memory Card Flash ID (serial number)
#define	sysFtrCreator			'psys'	// Creator type for System files

#define	sysFtrNumROMVersion	    1	// ROM Version

#define	sysFtrNumProcessorID	2	// Product ID
#define	sysFtrNumProcessorMask	0xFFFF0000	// Mask to obtain processor model
#define	sysFtrNumProcessor328	0x00010000	// Motorola 68328	(Dragonball)
#define	sysFtrNumProcessorEZ	0x00020000	// Motorola 68EZ328	(Dragonball EZ)
#define	sysFtrNumProcessorVZ	0x00030000	// Motorola 68VZ328	(Dragonball VZ)


/* CONSOLE MODE */

/* SerialLinkMgr.h */
//----------------------------------------------------------------------
// Socket IDs
//----------------------------------------------------------------------
#define		slkSocketDebugger			0			// Debugger Socket
#define		slkSocketConsole			1			// Console Socket

//----------------------------------------------------------------------
//  Packet Types
//----------------------------------------------------------------------
#define		slkPktTypeSystem			0			// System packets

//----------------------------------------------------------------------
// Packet header
// Fields marked with -> must be filled in by caller
// Fields marked with X  will be filled in by SlkSendPacket.
//----------------------------------------------------------------------
typedef struct SlkPktHeaderType {
	unsigned short	signature1;	// X  first 2 bytes of signature
	unsigned char	signature2;	// X  3 and final byte of signature
	unsigned char	dest;		// -> destination socket ID
	unsigned char	src;		// -> src socket ID
	unsigned char	type;		// -> packet type
	unsigned short	bodySize;	// X  size of body
	unsigned char	transId;	// -> transaction ID (if 0 specified, it will be replaced)
	unsigned char	checksum;	// X  check sum of header
	} SlkPktHeaderType;

#define	slkPktHeaderSignature1	0xBEEF
#define	slkPktHeaderSignature2	0xED

//----------------------------------------------------------------------
// Packet footer
//----------------------------------------------------------------------
typedef struct SlkPktFooterType {
	unsigned short	crc16;	// header and body crc
	} SlkPktFooterType;

/* SystemPkt.h */
//-----------------------------------------------------------------------
// Packet body structure
//-----------------------------------------------------------------------
// Common fields at the start of every body
#define	_sysPktBodyCommon		\
	unsigned char	command;	\
	unsigned char	_filler

// Generic System Packet Body
typedef struct SysPktBodyType {
	_sysPktBodyCommon;
	unsigned char	data[sysPktMaxBodySize-2];
	} SysPktBodyType;


//--------------------------------------------------------------------
// Read Memory command
//--------------------------------------------------------------------
#define	sysPktReadMemCmd				0x01
#define	sysPktReadMemRsp				0x81
			
typedef struct SysPktReadMemCmdType {
	_sysPktBodyCommon;					// Common Body header
	unsigned int			address;	// Address to read
	unsigned short			numBytes;	// # of bytes to read
	} SysPktReadMemCmdType;


//--------------------------------------------------------------------
// Remote Procedure call
//--------------------------------------------------------------------
#define	sysPktRPCCmd					0x0A
#define	sysPktRPCRsp					0x8A

typedef struct SysPktRPCParamInfo {
	unsigned char		byRef;		// true if param is by reference
	unsigned char		size;		// # of Bytes of paramData	(must be even)			
	unsigned short		data[1];	// variable length array of paramData
	} SysPktRPCParamType;
	
typedef struct SysPktRPCType {
	_sysPktBodyCommon;				// Common Body header
	unsigned short		trapWord;	// which trap to execute
	unsigned int		resultD0;	// result from D0 placed here
	unsigned int		resultA0;	// result from A0 placed here
	unsigned short		numParams;	// how many parameters follow
	// Following is a variable length array ofSlkRPCParamInfo's
	SysPktRPCParamType	param[1];
	} SysPktRPCType;


/* CoreTraps.h */
//--------------------------------------------------------------------
// Palm OS Core Trap numbers
//--------------------------------------------------------------------
#define sysTrapMemCardInfo		0xA004
#define sysTrapSysReset			0xA08C
#define sysTrapFtrGet			0xA27B
#define sysTrapHwrGetROMToken	0xA340


/* FUNCTION DEFINITIONS */

/* main.c */

void	usage (char *argv);
void	help (void);

/* crc.c */

BOOL            verify_checksum (void);
unsigned short	calc_crc16 (const void* bufP, unsigned short count, unsigned short crc);
unsigned char	calc_hdr_checksum (unsigned char start, unsigned char* bufP, int count);

/* serial.c */

int		open_serial_port (LPCTSTR gszPort, DWORD t);
void	close_serial_port (void);
BOOL	write_buffer (char* lpBuf, DWORD dwToWrite);
BOOL	read_buffer (char* lpBuf, DWORD dwToRead);
void	flush_buffer (char* lpBuf);

/* pdd.c */

void    wrapup (void);
void	pdd (char* port, char* filename, unsigned short cardNo, MemoryType memtype);
void	pdd_ReadMemory (unsigned int address, unsigned short numBytes);
void	pdd_SysReset (void);
void	pdd_FtrGet(unsigned int creator, unsigned short featureNum, unsigned int *valueP);
void	pdd_HwrGetROMToken (unsigned short cardNo, unsigned int token, unsigned char *data);
void	pdd_MemCardInfo (unsigned short cardNo, unsigned char *cardName, unsigned char *manufName, 
						 unsigned short *version, unsigned int *crDate,  unsigned int *romSize, 
						 unsigned int *ramSize, unsigned int *freeBytes);
int		create_packet (unsigned char dest, unsigned char src, unsigned char type, 
					   unsigned char transId, void* dataP, unsigned short bodySize);
void	parse_packet (char* lpBuf);


/* GLOBAL VARIABLES */

extern unsigned char	rxbuffer[MAX_BUF_SIZE];
extern SlkPktHeaderType	header;	
extern SlkPktFooterType	footer;		

#pragma pack(pop)

#endif
