/*
	serial.c
 
    This file is part of pdd (Palm dd).
 
    A memory imaging tool for forensic analysis of Palm OS platform devices

    Copyright (c) 2002 Joe Grand
*/

#include "pdd.h"

static HANDLE hComm;	/* serial port */

int open_serial_port (LPCTSTR gszPort, DWORD t)
{
	DCB dcb;
	COMMTIMEOUTS timeout;

	/* Open serial port */
	hComm = CreateFile(gszPort, GENERIC_READ | GENERIC_WRITE, 0, 0, OPEN_EXISTING, 0, 0);
	if (hComm == INVALID_HANDLE_VALUE)  // error opening port; abort
		return (1);

	/* Configure serial port */
	GetCommState(hComm, &dcb);
    GetCommTimeouts(hComm, &timeout);
	if (!BuildCommDCBAndTimeouts("baud=57600 parity=N data=8 stop=1 to=on", &dcb, &timeout)) 
		return (1);

    /* Set timeout values */
    timeout.ReadIntervalTimeout = 0;
    timeout.ReadTotalTimeoutConstant = t;
    timeout.ReadTotalTimeoutMultiplier = 0;
	timeout.WriteTotalTimeoutConstant = t;
    timeout.WriteTotalTimeoutMultiplier = 0;
	if (!SetCommState(hComm, &dcb) || !SetCommTimeouts(hComm, &timeout))
		return (1);

	return (0);
}


void close_serial_port (void)
{
	CloseHandle (hComm);
}


BOOL write_buffer (char* lpBuf, DWORD dwToWrite)
{
	DWORD dwWritten;
	BOOL fRes = TRUE;

	// Issue write.
	if (!WriteFile(hComm, lpBuf, dwToWrite, &dwWritten, NULL)) 
	{
      if (GetLastError() != ERROR_IO_PENDING)
         // WriteFile failed, but it isn't delayed. Report error and abort.
         return (FALSE);
	}

	return (dwWritten);
}


BOOL read_buffer (char* lpBuf, DWORD dwToRead)
{
	DWORD dwRead;

	// Issue read operation.
	if (!ReadFile(hComm, lpBuf, dwToRead, &dwRead, NULL)) 
	{
		if (GetLastError() != ERROR_IO_PENDING)
			// Error in communications; report it.
			return (FALSE);
	}
	
	return (dwRead);
}


void flush_buffer (char* lpBuf)
{
	memset (lpBuf, 0, MAX_BUF_SIZE);
}
